/*********************************************************************
 *
 * File:         global.h
 *
 * Description:  Global definitions for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <winsock.h>

//#define QUIET

#ifndef QUIET
#define PRINTF printf
#else
#define PRINTF nic 
#endif

#define MAX_UDP_SIZE 60000

#define MAX_MEMBERS 20
#define MAX_APPS 20

#define DAEMON_PORT 5000
#define MANAGER_PORT 4999
#define SERVER_PORT 4998

#define MSG_HEADER_LENGTH 56
#define MEMBER_LENGTH 25

typedef struct _participant
{
  unsigned char handle;
  char user[20];
  int host;
} Participant;

typedef struct _msg_header
{
  unsigned char tag;
  unsigned char session;
  Participant sender;
  Participant receiver;
  int dataLength;
} MsgHeader;

/* Tags in local messages */
typedef enum _local_tag
{
  /* local requests */
  reqLocalSessionCreate,          
  reqLocalSessionQuit,
  reqLocalSessionAddMember,
  reqLocalSessionJoin,
  reqLocalSessionReject,
  reqLocalSessionGetMembers,      
  reqLocalSessionSendToApp,
  reqLocalSessionStartApp,
  reqLocalSessionReset,

  reqLocalSessionSaveInit,
  reqLocalSessionSaveClose,
  reqLocalSessionRestoreInit,
  reqLocalSessionRestoreClose,

  /* local indications */
  indLocalSessionInvited,
  indLocalSessionAccept,
  indLocalSessionReject,          
  indLocalSessionMembers,
  indLocalSessionAddMember,
  indLocalSessionDeleteMember,
  indLocalSessionLocalMember,
  indLocalSessionStartApp,
  indLocalSessionTimeout,
  indLocalSessionConnected,

  indLocalAppQuit,
  indDbStart,
  indDbStop
} LocalTag;

typedef enum _member_tag
{
  /* remote requests */
  reqRemoteSessionAccept,
  reqRemoteSessionQuit,
  reqRemoteSessionReject,
  reqRemoteSessionAddMember,
  reqRemoteSessionAddRest,
  reqRemoteSessionSendToApp,
  reqRemoteSessionSendAppData,
  reqRemoteSessionStartApp,

  /* remote acknowledgments */
  ackRemoteInvitationReceived,

  /* server messages */
  reqServerAddUser,
  reqServerDelUser,
  reqServerUsers,
  indServerUsers
} MemberTag;

typedef enum _app_tag
{
  reqAppDettach=50,
  reqAppData,
  indAppData
} AppTag;

typedef enum _new_tag
{
  reqNewLocalSessionInit=60,
  reqNewRemoteSessionConnect,
  reqNewRemoteSessionInvite,
  reqNewLocalApplicationAttach
} NewTag;

typedef struct _invite_data
{
  int handle;
  int numMembers;
  Participant parts[1];
} InviteData;


typedef struct _member_info
{
  int handle;
  Participant part;
  struct sockaddr_in addr;
} MemberInfo;

typedef struct _app_data
{
  unsigned char handle;
  int recID;
  int port;
  short tag;
  char data[1];
} AppData;

enum _db_state
{
  DbStateIdle,
  DbStateSave,
  DbStateRestore
};

#define LOCAL 1
#define REMOTE 0

void nic();

int sendControlMessage(int sock, MsgHeader *header, char *msg, int len);
int gethostid();
int getHostFullName(char *name, int len);
int getHostName(int id, char *host);
int writeAll(int fd, void* buffer, size_t len);
int sendControlMessage(int sock, MsgHeader *header, char *msg, int len);
int sendLocalMessage(short tag, char *msg, int len);
int sendMemberMessage(int no, short tag, char *msg, int len);
int connect2CE(int hostid);
int sendNewMessage(Participant *part, short tag, 
			     char *msg, int len);
int connect2member(int hostid);
int sendAppMessage(int no, short tag, char *msg, int len);
int sendInviteMessage(short tag, char *msg, int len);
void getStartPath(char *path);

void localClbSessionCreate(MsgHeader *header, char *msg, int len);
void localClbSessionAddMember(MsgHeader *header, char *msg, int len);
void localClbSessionJoin(MsgHeader *header, char *msg, int len);
void localClbSessionReject(MsgHeader *header, char *msg, int len);
void localClbSessionGetMembers(MsgHeader *header, char *msg, int len);
void localClbSessionQuit(MsgHeader *header, char *msg, int len);
void localClbSessionSendToApp(MsgHeader *header, char *msg, int len);
void localClbSessionStartApp(MsgHeader *header, char *msg, int len);
void localClbSessionSaveInit(MsgHeader *header, char *msg, int len);
void localClbSessionSaveClose(MsgHeader *header, char *msg, int len);
void localClbSessionRestoreInit(MsgHeader *header, char *msg, int len);
void localClbSessionRestoreClose(MsgHeader *header, char *msg, int len);
void localClbSessionReset(MsgHeader *header, char *msg, int len);
void newClbSessionInit(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len);
void newClbSessionConnect(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len);
void newClbAppAttach(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len);
void newClbSessionInvite(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len);
void appClbAppDettach(int no, MsgHeader *header, char *msg, int len);
void appClbAppData(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionAddRest(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionAccept(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionReject(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionAddMember(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionQuit(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionSendToApp(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionSendAppData(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionStartApp(int no, MsgHeader *header, char *msg, int len);

