/*********************************************************************
 *
 * File:         session_manage.h
 *
 * Description:  Header of session management module for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-15-96
 *
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h>

typedef enum _session_status
{
  statusNone,
  statusInit,
  statusCreated,
  statusAbandoned
} SessionStatus;

typedef struct _session
{
  SessionStatus status;
  int id;

  int memberActive[MAX_MEMBERS];
  struct sockaddr_in memberAddr[MAX_MEMBERS];
  Participant member[MAX_MEMBERS];
  int lastMember;
  int numMembers;
  int localMember;

  int appActive[MAX_APPS];
  int appPort[MAX_APPS];
  struct sockaddr_in appAddr[MAX_APPS];
  int lastApp;
  int numApps;
} Session;


int addr2name(struct sockaddr_in *addr, char *name);
int start_process(char *path, PROCESS_INFORMATION *pinfo);
void startCM();
int newMember(Participant *member);

int sessionInit();
void setSessionID(int id);
int sessionCreate(Participant *initiator);
void sessionAbandon();

int sessionAddMember(Participant *newmember);
void sessionDeleteMember(int no);
void sessionSetLocalMember(int no);

int sessionAddApp(int no, unsigned short port);
void sessionDeleteApp(int no);

int sessionGetNoMembers();
void sessionGetMembers(Participant *parts);
int getReceiverData(int userNo, MsgHeader *header);
int getUserData(int userNo, struct sockaddr_in *addr, Participant *part);
int getUserHandle(Participant *part);
int getLocalHandle();

int sessionCreated();
int sessionInitialized();
int sessionAbandoned();
int getSessionID();

int getAppNumber(int port);











