///////////////////////////////////////////////////////////////////////////
// 
// File						: Conference.cpp
//
// Description				: Conference functions implementation
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-15-96
//
////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "ncs.h"
#include "gsm.h"
#include "DSaudio.h"
#include "TSaudioProc.h"
#include "TSaudioDlg.h"
#include "Conference.h"

#define IDM_EXIT                        32771

CTSaudioDlg *global_dlg;
Conference *cf;

void Exit()
{
	SendMessage(global_dlg->m_hWnd, WM_CLOSE, 0, NULL); 
}

Conference::Conference(CTSaudioDlg *dlg)
{
	int i;
	HANDLE hProcess;

	main_dlg = dlg;
	global_dlg = dlg;
  
	hProcess = GetCurrentProcess();
	SetPriorityClass(hProcess, HIGH_PRIORITY_CLASS);
	
	code_type = ADPCM;
	mode = DUPLEX;
	treshold = 2000;
	timer = 0;
	silence_no = 0;
	silence_mic = 0;
	signal = 0;
	current_sender = -1;
	talk = 1;

	// ADPCM init
	adpcmsendstate.valprev = 0;
	adpcmsendstate.index = 0;

	for(i=0; i<MAX_USERS; i++)
	{
		adpcmstates[i].valprev = 0;
		adpcmstates[i].index = 0;
		first[i] = 1;
    }

	gsmHandle = gsm_create();
	
	// Start up NCS
	ncsApp = (ncsApplication *)ncsAppAttach(AUDIO_PLUGIN_PORT);
	if(ncsApp == NULL)
	{
		Exit();
		return;
	}

	ncsAppAddCallback(ncsApp, tagAppQuit, (void (*)())Exit);

	if(!hardwareInit())
		SendMessage(main_dlg->m_hWnd, WM_CLOSE, 0, NULL); 


	if(mode == HALF_DUPLEX)
	{
		automatic = 1;
		main_dlg->m_Auto.SetCheck(1);
		state = stateMic;
	}
	else
	{
		automatic = 1;
		main_dlg->m_Auto.SetCheck(1);
		state = stateBoth;
		setMic();
	}

	// Create reading thread
	hThread = CreateThread( 
		    NULL,									// no security attributes 
			0,										// use default stack size  
			(LPTHREAD_START_ROUTINE) ReadThread,	// thread function 
			this,									// argument to thread function 
			0,										// use default creation flags 
			&dwThreadId);							// returns the thread identifier 
	// SetThreadPriority(hThread, THREAD_PRIORITY_TIME_CRITICAL);
	
	if (hThread == NULL) 
	{
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	}

	cf = this;

}

int Conference::hardwareInit()
{
	// audio init
	audioInit(this);
	if(!audioWriteInit())
	{
		main_dlg->MessageBox("There is no appropriate audio device", "Info", MB_OK | MB_ICONEXCLAMATION);
		return 0;
	}

	audioWriteClose();

	if(!audioReadInit())
	{
		main_dlg->MessageBox("Audio device does not support audio capturing", "Info", MB_OK | MB_ICONEXCLAMATION);
		audioWriteInit();
	}
	else
		if(!audioWriteInit())
		{
			mode = HALF_DUPLEX;
			main_dlg->MessageBox("Audio device supports only HALF DUPLEX mode", "Info", MB_OK | MB_ICONEXCLAMATION);
			main_dlg->disableControl();
		}

	return 1;
}


DWORD Conference::ReadThread(LPDWORD lpdwParam)
{
	struct timeval timeout;
	fd_set set;
	ncsData *ncsdata;
	int size;
	int sockfd;
	char *data;
	ncsApplication *app = ((Conference *)lpdwParam)->ncsApp;

	timeout.tv_sec = 0;
	timeout.tv_usec = 1000;

	while(1)
	{
		sockfd = ncsAppGetFD(app, &set);
		if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
		{
			ncsAppProcessIncoming(app, &set);
			size = ncsAppGetData(app, &set, &data);
			if(size > 0)
			{
				size = size - sizeof(ncsData) + 1;
				ncsdata = (ncsData *)data;
				ncsdata->handle = ntohl(ncsdata->handle);
				ncsdata->type = ntohl(ncsdata->type);
				((Conference *)lpdwParam)->HandleData(ncsdata->type, ncsdata->data, size, 
								       ncsdata->handle);
				free(data);
			}
		}
	}
	return 1;
}

void Conference::HandleData(DWORD tag, void *data, int len, DWORD handle)
{
	char buffer[1600];
	short level;
	int i;
	
	// MessageBox(NULL, "Get audio data", "Info", MB_OK);
	switch (tag)
	{
	case ADPCM:
		//if((mode == DUPLEX) || (state == stateStop))
		//{
			adpcm_decoder((char *)data, (short *)buffer, 800, &adpcmstates[handle]);
		//}
		break;

    case GSM:
		for(i=0; i<5; i++)
		{
			gsm_decode(gsmHandle, ((gsm_frame *)data)[i], (short *)&buffer[i * 320]);
		}

		break;

	case PCM:
		memcpy(buffer, data, 1600);
		for(i=0; i<800; i++)
			((short*)buffer)[i] = ntohs(((short *)buffer)[i]);
		break;
	}
	
	level = audioGetLevel(buffer);
	main_dlg->m_LedOut.SetPos(level/330);
	//level = 0;
	
	switch(state)
	{
	case stateMic:
		if(level > treshold)
		{
			signal = 1;
			//stateControl(sig);
			//current_sender = handle;
			//audioWriteChunk(buffer);
		}
		break;

	case stateSpeaker:
/*
		if((current_sender == -1) && (level > treshold))
			current_sender = handle;

		if(handle == current_sender)
		{
			checkTimer();
			audioWriteChunk(buffer);
			if(level <= treshold)
			{
				silence_no++;
				if(silence_no == 10)
				{
					stateControl(Nsig);
					silence_no = 0;
					signal = 0;
				}
			}
			else
				silence_no = 0;
		}
*/
		
		if(level > treshold)
			checkTimer();
		dsaudio.play(handle, buffer);

		break;

	case stateBoth:
		if((current_sender == -1) && (level > treshold))
			current_sender = handle;

		if(handle == current_sender)
		{
			checkTimer();
			audioWriteChunk(buffer);
			if(level <= treshold)
			{
				silence_no++;
				if(silence_no == 10)
				{
					current_sender = -1;
					silence_no = 0;
					signal = 0;
				}
			}
			else
				silence_no = 0;
		}			
		break;
	}	
}

void Conference::SendData(char *data)
{
   audioPluginMessageStructure *msg;
   gsm_frame *gsm_msg;
   short *buf;
   int i;
   short level;

	level = audioGetLevel(data);
   switch(code_type)
	{
	case GSM:
		gsm_msg = (gsm_frame *)malloc(5 * sizeof(gsm_frame));
		for(i=0; i<5; i++)
		{
			gsm_encode(gsmHandle, (gsm_signal *)(&data[320 * i]), gsm_msg[i]);
		}
		ncsAppSendData(ncsApp, -1, GSM, gsm_msg, 5*sizeof(gsm_frame));
		free(gsm_msg);
	    break;
	      
	case ADPCM:
		msg = (audioPluginMessageStructure *)malloc(sizeof(audioPluginMessageStructure));
		if (!msg)
		{
			fprintf(stderr, "Message allocation error\n");
			return;
		}
		// MessageBox(NULL, "Sending audio data", "Info", MB_OK);
		adpcm_coder((short *)data, msg->buf, 800, &adpcmsendstate);
		ncsAppSendData(ncsApp, -1, ADPCM, msg, sizeof(audioPluginMessageStructure));
		free(msg);
		break;

	case PCM:
		buf = (short *)malloc(1600);
		memcpy(buf, data, 1600);
		for(i=0; i<800; i++)
			buf[i] = htons(buf[i]);
		ncsAppSendData(ncsApp, -1, PCM, buf, 1600);
		free(buf);
		break;

	}
   	
   main_dlg->m_LedIn.SetPos(level/330);

	if(level <= treshold)
	{
		silence_mic++;
		if(silence_mic >= 10)
		{
			checkTimer();
			if(signal)
			{
				silence_mic = 0;
				stateControl(sig);
			}
		}
	}
	else
		silence_mic = 0;

}



void Conference::setTreshold(UINT level)
{
	treshold = level * 327;
}

void Conference::codeADPCM(void)
{
	code_type = ADPCM;
}

void Conference::codeGSM(void)
{
	code_type = GSM;
}

void Conference::codePCM()
{
	code_type = PCM;
}

Conference::~Conference(void)
{
	TerminateThread(hThread, 0);
	CloseHandle(hThread);

	switch(state)
	{
	case stateMic:
		audioReadClose();
		break;

	case stateSpeaker:
		//audioWriteClose();
		dsaudio.close();
		break;

	case stateBoth:
		audioReadClose();
		audioWriteClose();
		break;
	}

	gsm_destroy(gsmHandle);
	audioClose();
	ncsAppDettach(ncsApp);
}

void Conference::setAuto(int check)
{
	stateControl(Abut);
}

void Conference::setTalk(int check)
{
	talk = check;
	stateControl(Sbut);
}

void Conference::checkTimer()
{
	if(timer != 0)
		KillTimer(global_dlg->m_hWnd, 23000);

	timer = SetTimer(global_dlg->m_hWnd, 23000, 1000, TimerCallback);
	if(timer == 0)
		MessageBox(NULL, "Timer creating error!", "Error", MB_OK);
}

VOID CALLBACK Conference::TimerCallback(
    HWND hwnd,	// handle of window for timer messages 
    UINT uMsg,	// WM_TIMER message
    UINT idEvent,	// timer identifier
    DWORD dwTime 	// current system time
   )
{
	cf->stateControl(Nsig);
	cf->signal = 0;
}


///////////////////////////////////////////////////////////////////////////
// 
// Automaton state transition
//
///////////////////////////////////////////////////////////////////////////
void Conference::stateControl(StateInput in)
{
	if(mode == DUPLEX)
	{
		FDstateControl(in);
	}

	if(mode == HALF_DUPLEX)
	{
		HDstateControl(in);
	}
}

void Conference::setMic()
{
	main_dlg->m_Talk.EnableWindow();
	main_dlg->autoTalk();

	switch(state)
	{
	case stateMic:
		break;

	case stateSpeaker:
		main_dlg->m_LedOut.SetPos(0);
		dsaudio.close();
		//audioWriteClose();
		audioReadInit();
		audioReadStart();
		break;

	case stateBoth:
		state = stateMic;
		audioWriteClose();
		break;
	}

	state = stateMic;
}

void Conference::setSpeaker()
{
	main_dlg->m_Talk.EnableWindow();
	main_dlg->autoNotTalk();
	
	switch(state)
	{
	case stateMic:
		audioReadClose();
		audioReadStop();
		main_dlg->m_LedIn.SetPos(0);
		//audioWriteInit();
		dsaudio.init(main_dlg->m_hWnd);
		break;

	case stateSpeaker:
		break;

	case stateBoth:
		audioReadClose();
		audioReadStop();
		break;
	}

	state = stateSpeaker;
}

void Conference::setBoth()
{
	main_dlg->m_Talk.EnableWindow(FALSE);
	main_dlg->autoTalk();

	switch(state)
	{
	case stateMic:
		audioWriteInit();
		break;

	case stateSpeaker:
		dsaudio.close();
		audioWriteInit();
		audioReadInit();
		audioReadStart();
		break;

	case stateBoth:
		break;
	}

	state = stateBoth;
}

void Conference::FDstateControl(StateInput in)
{
	if(automatic)
	{
		if(state == stateMic)
		{
			if(in == Abut)
			{
				automatic = 0;
				setBoth();
			}
			
			if((in == sig) || (in == Sbut))
			{
				setSpeaker();
			}
		}
		else
		if(state == stateSpeaker)
		{
			if(in == Abut)
			{
				automatic = 0;
				setBoth();
			}
			
			if((in == Nsig) || (in == Sbut))
			{
				setMic();
			}
		}
	}
	else
	{
		if(in == Abut)
		{
			automatic = 1;
			setMic();
		}
	}
}

void Conference::HDstateControl(StateInput in)
{
	if(automatic)
	{
		if(state == stateMic)
		{
			if(in == Abut)
			{
				automatic = 0;
			}
			
			if((in == sig) || (in == Sbut))
			{
				setSpeaker();
			}
		}
		else
		if(state == stateSpeaker)
		{
			if(in == Abut)
			{
				automatic = 0;
			}
			
			if((in == Nsig) || (in == Sbut))
			{
				setMic();
			}
		}
	}
	else
	{
		if(state == stateMic)
		{
			if(in == Abut)
			{
				automatic = 1;
			}
			
			if(in == Sbut)
			{
				setSpeaker();
			}
		}
		else
		if(state == stateSpeaker)
		{
			if(in == Abut)
			{
				automatic = 1;
				setMic();
			}
			
			if(in == Sbut)
			{
				setMic();
			}
		}
	}
}

