///////////////////////////////////////////////////////////////////////////
// 
// File						: Conference.h
//
// Description				: Conference functions declarations
//
// Author					: Tomasz Stachowiak
//
// Date						: 11-15-96
//
////////////////////////////////////////////////////////////////////////////
#ifndef _CONFERENCE_H
#define _CONFERENCE_H

#include "ncs.h"
#include "adpcm.h"
#include "gsm.h"
#include "DSaudio.h"
#include "TSaudio.h"
#include "TSaudioDlg.h"

#define		AUDIO_PLUGIN_PORT      12567
#define         VERSION      		1

#define MAX_USERS 100

#define ADPCM 13
#define GSM 14
#define PCM 15

#define DUPLEX 20
#define HALF_DUPLEX 21

#define TRESHOLD 2000

typedef struct _audioMessage
{
  char buf[400];
}  audioPluginMessageStructure;



typedef enum _states
{
	stateMic,
	stateSpeaker,
	stateBoth
} States;

typedef enum _state_input
{
	Abut,
	Sbut,
	sig,
	Nsig
} StateInput;

class Conference
{
private:
	void stateControl(StateInput in);
	void HDstateControl(StateInput in);
	void FDstateControl(StateInput in);
	void setBoth();
	void setSpeaker();
	void setMic();
	void checkTimer();
	ncsApplication *ncsApp;
	States state;
	int code_type;
	HANDLE hThread;
	DWORD dwThreadId;
	struct adpcm_state adpcmstates[MAX_USERS];
	struct adpcm_state adpcmsendstate;
	int mode;
	int first[MAX_USERS];
	CTSaudioDlg *main_dlg;
	int silence_no;
	int silence_mic;
	int signal;
	gsm gsmHandle;
	int current_sender;
	int treshold;

	DSmultiAudio dsaudio;

	void HandleData(DWORD tag, void *data, int len, DWORD handle);

public:
	void codePCM();
	UINT timer;
	void setTalk(int check);
	void setAuto(int check);
	Conference(CTSaudioDlg *dlg);
	int hardwareInit();
	static DWORD ReadThread(LPDWORD lpdwParam);
	void SendData(char *data);
	void codeADPCM(void);
	void codeGSM(void);
	void setTreshold(UINT level);
	~Conference(void);
	VOID static CALLBACK TimerCallback(
		HWND hwnd,	// handle of window for timer messages 
		UINT uMsg,	// WM_TIMER message
		UINT idEvent,	// timer identifier
		DWORD dwTime 	// current system time
	);
	
protected:
	int automatic;
	int talk;
};

#endif
