#include "stdafx.h"

#include <dsound.h>

#include "DSaudio.h"



int DSbuffer::init(LPDIRECTSOUND lpDirectSound, WAVEFORMATEX *pformat)
{
	HRESULT hr;
	
    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC)); // Zero it out.
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
    // Need default controls (pan, volume, frequency).
    dsbdesc.dwFlags = DSBCAPS_CTRLDEFAULT; 
    // 100-msecond buffer.
    dsbdesc.dwBufferBytes = 3 * pformat->nAvgBytesPerSec; 
    dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)pformat;

    // Create buffer.
    hr = lpDirectSound->CreateSoundBuffer(&dsbdesc, &DSbuf, NULL);
    if(DS_OK == hr) {
        // Succeeded. Valid interface is in *lplpDsb.
		DSbuf->SetVolume(0);

		return 1;
    } else {
        // Failed.
        DSbuf = NULL;
        return 0;
    }
}

int DSbuffer::play(void *buf)
{
    LPVOID lpvPtr1;
    LPVOID lpvPtr2;
    DWORD dwBytes1; 
    DWORD dwBytes2;
    HRESULT hr;
	char *tmpbuffer;

    // Obtain write pointer.
    hr = DSbuf->Lock(0, 3200, &lpvPtr1, 
        &dwBytes1, &lpvPtr2, &dwBytes2, DSBLOCK_FROMWRITECURSOR);

    // If DSERR_BUFFERLOST is returned, restore and retry lock.
    if(DSERR_BUFFERLOST == hr) {

        DSbuf->Restore();
        hr = DSbuf->Lock(0, 3200, &lpvPtr1, &dwBytes1, &lpvPtr2, &dwBytes2, DSBLOCK_FROMWRITECURSOR);
    }
    if(DS_OK == hr) {
        // Write to pointers.
		tmpbuffer = (char *)GlobalAlloc(GPTR, 3600);
		CopyMemory(tmpbuffer, buf, 1600);
		CopyMemory(lpvPtr1, tmpbuffer, dwBytes1);
		CopyMemory(lpvPtr2, &tmpbuffer[dwBytes1], dwBytes2);	
		GlobalFree(tmpbuffer);
	}

	// Release the data back to DirectSound.
    hr = DSbuf->Unlock(lpvPtr1, dwBytes1, lpvPtr2, dwBytes2);
    if(DS_OK == hr) {
        // Success.
		hr = DSbuf->Play(0, 0, 0/*DSBPLAY_LOOPING */);
		if(hr != DS_OK)
			return 0;
		return 1;
	}
    // Lock, Unlock, or Restore failed.
    return 0;
}

void DSbuffer::close()
{
	DSbuf->Stop();
	DSbuf->Release();
}


void DSmultiAudio::setFormat()
{
    memset(&format, 0, sizeof(WAVEFORMATEX));
    format.wFormatTag = WAVE_FORMAT_PCM;
	format.nChannels = 1;
	format.nSamplesPerSec = 8000;
    format.nAvgBytesPerSec = 16000; 
    format.nBlockAlign = 2; 
	format.wBitsPerSample = 16;
}

int DSmultiAudio::createPrimaryBuf()
{
	HRESULT hr;
	
	setFormat();
	
	memset(&dsbdesc, 0, sizeof(DSBUFFERDESC)); // Zero it out.
    dsbdesc.dwSize = sizeof(DSBUFFERDESC);
    dsbdesc.dwFlags = DSBCAPS_PRIMARYBUFFER;
    // Buffer size is determined by sound hardware.
    dsbdesc.dwBufferBytes = 0; 
    dsbdesc.lpwfxFormat = NULL; // Must be NULL for primary buffers.

	hr = lpDirectSound->CreateSoundBuffer(&dsbdesc, &DSprimaryBuffer, NULL);
    if(DS_OK == hr) {
		// Succeeded. Set primary buffer to desired format.
		hr = DSprimaryBuffer->SetFormat(&format);
		if(DS_OK == hr) {
			// If you want to know the buffer size, call GetCaps.
			/*
			dsbcaps.dwSize = sizeof(DSBCAPS);

			(*lplpDsb)->lpVtbl->GetCaps(*lplpDsb, &dsbcaps);
			*lpdwBufferSize = dsbcaps.dwBufferBytes;
			*/
			return 1;
		}
    }

	return 0;
}

int DSmultiAudio::init(HWND hwnd)
{
	int i;
	
	if(DS_OK == DirectSoundCreate(NULL, &lpDirectSound,
		NULL)) {
		// Creation succeeded.
		lpDirectSound->SetCooperativeLevel(hwnd, DSSCL_PRIORITY);

		if(!createPrimaryBuf())
			return 0;
		
		for(i=0; i<MAX_STREAMS; i++)
			buffer[i] = NULL;

		return 1;

	} else {
		// Creation failed.
		return 0;
	}	
	
	return 0;
}

void DSmultiAudio::close()
{
	int i;

	for(i=0; i<MAX_STREAMS; i++)
		if(buffer[i] != NULL)
			closeBuf(i);

	lpDirectSound->Release();
}


int DSmultiAudio::play(int id, void *buf)
{
	if(buffer[id] == NULL)
	{
		buffer[id] = new DSbuffer();
		if(!buffer[id]->init(lpDirectSound, &format))
		{
			delete buffer[id];
			buffer[id] = NULL;
			return 0;
		}
	}

	return buffer[id]->play(buf);
}

void DSmultiAudio::closeBuf(int id)
{
	if(buffer[id] != NULL)
	{
		buffer[id]->close();
		delete buffer[id];
		buffer[id] = NULL;
	}
}

