/*********************************************************************
 *
 * File:         global.h
 *
 * Description:  Global definitions for conference engine
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#ifndef _GLOBAL_H
#define _GLOBAL_H

#include <winsock.h>

#include "rtp.h"

//#define QUIET

//#ifndef QUIET
//#define PRINTF printf
//#else
//#define PRINTF nic 
//#endif

#ifdef _DEBUG
#define PRINTF fprintf
#else
#define PRINTF nic 
#endif

#define MAX_UDP_SIZE 60000

#define MAX_MEMBERS 20
#define MAX_APPS 20

#define DAEMON_PORT 5000
#define MANAGER_PORT 4999
#define SERVER_PORT 4998

#define MSG_HEADER_LENGTH 56
#define MEMBER_LENGTH 25

typedef struct _participant
{
  unsigned char handle;
  char user[20];
  int host;
} Participant;

typedef struct _NCS_participant
{
  char user[20];
  char host[50];
} NCSparticipant;

typedef struct _msg_header
{
  unsigned char tag;
  unsigned char session;
  Participant sender;
  Participant receiver;
  int dataLength;
} MsgHeader;

/* Tags in local messages */
typedef enum _local_tag
{
  /* local requests */
  reqLocalSessionCreate,          
  reqLocalSessionQuit,
  reqLocalSessionAddMember,
  reqLocalSessionJoin,
  reqLocalSessionReject,
  reqLocalSessionGetMembers,      
  reqLocalSessionSendToApp,
  reqLocalSessionStartApp,
  reqLocalSessionReset,

  reqLocalSessionSaveInit,
  reqLocalSessionSaveClose,
  reqLocalSessionRestoreInit,
  reqLocalSessionRestoreClose,

  /* local indications */
  indLocalSessionInvited,
  indLocalSessionAccept,
  indLocalSessionReject,          
  indLocalSessionMembers,
  indLocalSessionAddMember,
  indLocalSessionDeleteMember,
  indLocalSessionLocalMember,
  indLocalSessionStartApp,
  indLocalSessionTimeout,
  indLocalSessionConnected,

  indLocalAppQuit,
  indDbStart,
  indDbStop
} LocalTag;

typedef enum _member_tag
{
  /* remote requests */
  reqRemoteSessionAccept,
  reqRemoteSessionQuit,
  reqRemoteSessionReject,
  reqRemoteSessionAddMember,
  reqRemoteSessionAddRest,
  reqRemoteSessionSendToApp,
  reqRemoteSessionSendAppData,
  reqRemoteSessionStartApp,

  /* remote acknowledgments */
  ackRemoteInvitationReceived,

  /* server messages */
  reqServerAddUser,
  reqServerDelUser,
  reqServerUsers,
  indServerUsers
} MemberTag;

typedef enum _app_tag
{
  reqAppDettach=50,
  reqAppData,
  indAppData
} AppTag;

typedef enum _new_tag
{
  reqNewLocalSessionInit=60,
  reqNewRemoteSessionConnect,
  reqNewRemoteSessionInvite,
  reqNewLocalApplicationAttach
} NewTag;

typedef struct _invite_data
{
  int handle;
  int numMembers;
  Participant parts[1];
} InviteData;


typedef struct _member_info
{
  int handle;
  Participant part;
  struct sockaddr_in addr;
} MemberInfo;

typedef struct _app_data
{
  unsigned char handle;
  int recID;
  int port;
  short tag;
  char data[1];
} AppData;

enum _db_state
{
  DbStateIdle,
  DbStateSave,
  DbStateRestore
};

typedef struct _server_data
{
  int numMembers;
  Participant parts[1];
} ServerData;

typedef enum _session_status
{
  statusNone,
  statusInit,
  statusCreated,
  statusAbandoned
} SessionStatus;

typedef struct _session
{
  SessionStatus status;
  int id;

  int memberActive[MAX_MEMBERS];
  struct sockaddr_in memberAddr[MAX_MEMBERS];
  Participant member[MAX_MEMBERS];
  int lastMember;
  int numMembers;
  int localMember;
} Session;

typedef struct _ncs_application
{
  int port;
  int socket;
  int sessionID;
  struct sockaddr_in addr;
  struct sockaddr_in mcast_addr;
  Session *session;
} ncsApplication;

typedef struct _ncs_data
{
  int type;
  int handle;
  char data[1];
} ncsData;


#define LOCAL 1
#define REMOTE 0

void nic(...);

int sendControlMessage(int sock, MsgHeader *header, char *msg, int len);
int gethostid();
int getHostFullName(char *name, int len);
int getHostName(int id, char *host);
int getHostID(char *host);
int writeAll(int fd, void* buffer, size_t len);
int sendControlMessage(int sock, MsgHeader *header, char *msg, int len);
int sendLocalMessage(short tag, char *msg, int len);
int sendMemberMessage(int no, short tag, char *msg, int len);
int connect2CE(int hostid);
int sendNewMessage(Participant *part, short tag, 
			     char *msg, int len);
int connect2member(int hostid);
int sendAppMessage(int usr, int no, short tag, char *msg, int len);
int sendInviteMessage(short tag, char *msg, int len);

void ncsSessionCreate();
void ncsSessionAddMember(Participant *part);
void ncsSessionJoin();
void ncsSessionReject();
//void localClbSessionGetMembers(MsgHeader *header, char *msg, int len);
void ncsSessionQuit();
//void localClbSessionSendToApp(MsgHeader *header, char *msg, int len);
//void localClbSessionStartApp(MsgHeader *header, char *msg, int len);
//void localClbSessionSaveInit(MsgHeader *header, char *msg, int len);
//void localClbSessionSaveClose(MsgHeader *header, char *msg, int len);
//void localClbSessionRestoreInit(MsgHeader *header, char *msg, int len);
//void localClbSessionRestoreClose(MsgHeader *header, char *msg, int len);
void ncsSessionReset();

void ncsSessionInit();
void newClbSessionConnect(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len);
void newClbAppAttach(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len);
void newClbSessionInvite(struct sockaddr_in *addr, MsgHeader *header, char *msg, int len);
void appClbAppDettach(int no, MsgHeader *header, char *msg, int len);
void appClbAppData(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionAddRest(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionAccept(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionReject(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionAddMember(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionQuit(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionSendToApp(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionSendAppData(int no, MsgHeader *header, char *msg, int len);
void remoteClbSessionStartApp(int no, MsgHeader *header, char *msg, int len);

int ncsServerInit(char *sname);
void ncsServerAdd(void);
void ncsServerDelete(void);
char *ncsServerUsers();

ncsApplication *ncsAppAttach(int port, Session *ses);
void ncsAppDettach(ncsApplication *app);
int ncsAppSendData(ncsApplication *app, int mbr, int type, void *data, int len);
int ncsAppSendPriorityData(ncsApplication *app, int mbr, int type, void *data, int len);
int ncsAppProcessIncoming(ncsApplication *app, fd_set *set);
int ncsAppGetData(ncsApplication *app, fd_set *set, char **data);
int ncsAppGetFD(ncsApplication *app, fd_set *set);
int ncsAppGetLocalHandle(ncsApplication *app);
int ncsAppGetNumMembers(ncsApplication *app);
int ncsAppGetPart(ncsApplication *app, int no, char *name, char *host);
void readRTPheader(RTPheader *hdr, char *data);
void fillRTPheader(RTPheader *hdr, char *data);

#endif
