#include "stdafx.h"
#include "ManagerTango.h"
#include "ManagerTangoDlg.h"
#include "ncs.h"
#include "tango.h"
#include "Conference.h"

// conference data
int conferenceExists = 0;

// main dialog pointer
CManagerTangoDlg *main_dlg;

//reading thread data
HANDLE hReadThread;
DWORD dwReadThreadId;

// plugins data
PLUGIN plugin[20];
int numPlugs;

// info about daemon process
PROCESS_INFORMATION daemon_info;

SOCKET tangoSock;


////////////////////////////////////////////////////////////////////////
//
// Conference management
//
////////////////////////////////////////////////////////////////////////
int start_conference(CManagerTangoDlg *dlg)
{
	char path[100];
	int len;
	main_dlg = dlg;
	 
	main_dlg->SetStatus("Initializing...");

	getStartPath(path);
	if(strlen(path) > 0)
		strcat(path, "\\");
	strcat(path, DAEMON_PATH);

	startProcess(path, &daemon_info);

	Sleep(1500);

	//////////////////////////////////
	// NCS initialization            
	if(!ncsSessionInitialize())
	{
		MessageBox(NULL, "Session initialization error", "Error", MB_OK);
		return 0;
	}

	//////////////////////////////////
	// Callbacks setting
	ncsSessionAddCallback(indLocalSessionInvited, invited);
	ncsSessionAddCallback(indLocalSessionStartApp, start_app);
    ncsSessionAddCallback(indLocalSessionAccept, accepted);

	tangoSock = registerApp(main_dlg->tangoAID, main_dlg->tangoPort);

	//////////////////////////////////
	// Reading thread creation
	hReadThread = CreateThread(
        NULL,									// no security attributes 
        0,										// use default stack size  
        (LPTHREAD_START_ROUTINE) readingThread, // thread function 
        NULL,									// argument to thread function 
        0,										// use default creation flags 
        &dwReadThreadId);						// returns the thread identifier 
	if (hReadThread == NULL) 
		MessageBox(NULL, "CreateThread error", "Error", MB_OK);
	
	read_plugins();

	main_dlg->SetStatus("Connecting...");

	tangoAck();

	return 1;
}

void exit_conference(void)
{
	TerminateThread(hReadThread, 0);
	CloseHandle(hReadThread);
	quit_plugins();
	ncsSessionQuit();
	Sleep(2000);
	TerminateProcess(daemon_info.hProcess, 0);
}


void add_user(char *member)
{
	NCSparticipant *part;

	part = (NCSparticipant *)member;
	if(!conferenceExists)
	{
		ncsSessionCreate();
		conferenceExists = 1;
	}
	
	ncsSessionAddMember(part->user, part->host);
}

////////////////////////////////////////////////////////////////////////
//
// Applications management
//
////////////////////////////////////////////////////////////////////////
void read_plugins(void)
{
	FILE *file;
	int i, len;
	char path[100];
	char plugin_path[100];

	getStartPath(path);
	if(strlen(path) > 0)
		strcat(path, "\\");
	strcat(path, PLUGINS_CFG_PATH);

	if((file = fopen(path, "r")) == NULL)
		MessageBox(NULL, "Can't read file", "Error", MB_OK);
	else
	{
		fscanf(file, "%d\n", &numPlugs);
		for(i=0; i<numPlugs; i++)
		{
			fscanf(file, "%d\n", &plugin[i].port);
			fgets(plugin[i].name, 50, file);
			plugin[i].name[strlen(plugin[i].name)-1] = 0;
			getStartPath(plugin[i].path);
			fgets(plugin_path, 100, file);
			strcat(plugin[i].path, plugin_path);
			plugin[i].path[strlen(plugin[i].path)-1] = 0;
			plugin[i].started = 0;
			main_dlg->AddApp(plugin[i].name);
		}
	}
}


void quit_plugins(void)
{
	int i;
	
	for(i=0; i<numPlugs; i++)
	{
		if(plugin[i].started)
		{
			ncsSessionSendToApp(plugin[i].port, indLocalAppQuit, NULL, 0);
			plugin[i].started = 0;
		}
	//		TerminateProcess(plugin[i].info.hProcess, 0);
	}
}

void startProcess(char *path, PROCESS_INFORMATION *pinfo)
{
	STARTUPINFO sinfo;

	sinfo.cb = sizeof(STARTUPINFO);
	sinfo.lpReserved = NULL;
	sinfo.lpDesktop = NULL;
	sinfo.lpTitle = NULL;
	sinfo.dwFlags = STARTF_FORCEONFEEDBACK;
	sinfo.cbReserved2 = 0;
	sinfo.lpReserved2 = NULL;

	if(CreateProcess(path, NULL, NULL, NULL, FALSE, 
					NORMAL_PRIORITY_CLASS | DETACHED_PROCESS, NULL, NULL, 
					&sinfo, pinfo)
		== FALSE)
			MessageBox(NULL, "Create process error", "Error", MB_OK);
}


void create_app(int port)
{
	int found = 0, index, i;

	for(i=0; i<numPlugs; i++)
	{
		if(plugin[i].port == port)
		{
			index = i;
			found = 1;
		}
	}
	if((found) && (!plugin[index].started))
	{
		// MessageBox(NULL, plugin[index].path, "Info", MB_OK);
		ncsSessionStartApp(port);
		plugin[index].started = 1;
		startProcess(plugin[index].path, &plugin[index].info);

	}
}

void start_local_app(int index)
{
	create_app(plugin[index].port);
}

void quit_application(int port)
{
	int found = 0, index, i;

	for(i=0; i<numPlugs; i++)
	{
		if(plugin[i].port == port)
		{
			index = i;
			found = 1;
		}
	}
	if((found) && (plugin[index].started))
	{
		plugin[index].started = 0;
		ncsSessionSendToApp(port, indLocalAppQuit, NULL, 0);
		//		TerminateProcess(plugin[index].info.hProcess, 0);
	}
}

void quit_local_app(int index)
{
	if(plugin[index].started)
	{
		plugin[index].started = 0;
		ncsSessionSendToApp(plugin[index].port, indLocalAppQuit, NULL, 0);
		//TerminateProcess(plugin[index].info.hProcess, 0);
	}
}

void info_app(int index)
{
	char text[200];

	wsprintf(text, "Port: %d\nName: %s\nPath: %s\n", plugin[index].port,
													 plugin[index].name,
													 plugin[index].path);
	main_dlg->MessageBox(text, "Info", MB_OK | MB_ICONINFORMATION);
}

/////////////////////////////////////////////////////////////////////////
//
// NCS callbacks
//
/////////////////////////////////////////////////////////////////////////
void invited(void *data)
{
	NCSparticipant *part;

	part = (NCSparticipant *)data;
	
	if(!conferenceExists)
	{
		conferenceExists = 1;
	}
		
	ncsSessionAcceptInvitation(part);

	main_dlg->SetStatus("Ready to use");
}

void start_app(void *data)
{
	int *port;

	port = (int *)data;
	create_app(*port);
}

void accepted(void *data)
{
	main_dlg->SetStatus("Ready to use");
}

/////////////////////////////////////////////////////////////////////////
//
// Tango message handler
//
/////////////////////////////////////////////////////////////////////////
void handleTangoMsg(void)
{
  TangoMsg *msg;
  struct Event *event;
  
  event = receiveEvent(tangoSock);
  if(event == NULL)
  {
	  //MessageBox(NULL, "Manager: error reading Tango message, exiting...", "Error", MB_OK);
	  SendMessage(main_dlg->m_hWnd, WM_CLOSE, 0, NULL); 
	  return;
  }
  msg = (TangoMsg *)getData(event);
 switch(msg->tag)
    {
    case 1:
      /* send invitation */
	  // Sleep(10000);
	  add_user((char *)&msg->part);
      break;
      
    case 2:
      /* quit */
	  SendMessage(main_dlg->m_hWnd, WM_CLOSE, 0, NULL); 
      break;

    default:
      MessageBox(NULL, "CM: wrong tango message tag", "Error", MB_OK);
    }
  destroyEvent(event);
}

void tangoAck(void)
{
  struct Event *event;
  char tag = 100; 
  event = createEvent(CONTROL, 1, &tag); 
  sendEvent(tangoSock, event); 
  destroyEvent(event); 
}

/////////////////////////////////////////////////////////////////////////
//
// Reading thread
//
/////////////////////////////////////////////////////////////////////////
DWORD readingThread(LPDWORD data)
{
	struct timeval timeout;
	fd_set set;
	int sockfd;

	timeout.tv_sec = 0;
	timeout.tv_usec = 500000;
	while(1)
	{
		sockfd = ncsSessionGetFD(&set);
		FD_SET(tangoSock, &set);
		if(select(FD_SETSIZE, &set, 0, 0, &timeout) > 0)
		{
			ncsProcessIncoming(&set);
		
			/* handle Tango message */
			if(FD_ISSET(tangoSock, &set))
				handleTangoMsg();
		}
	}

	return 1;
}

void getStartPath(char *path)
{
	int i;
	char *tmp;
	int found = 0;
	char p[100];

	tmp = GetCommandLine();
	strcpy(p, tmp);
	i = strlen(p) - 1;
	while(!found && (i >= 0))
	{
		if(p[i] == '\\')
		{
			p[i] = 0;
			found = 1;
		}
		i--;
	}
	if(!found)
		strcpy(p, "");
	
	if(p[0] == '"')
		strcpy(path, &p[1]);
	else
		strcpy(path, p);

}

////////////////////////////////  END  /////////////////////////////////////