/*********************************************************************
 *
 * File:         sessions.c
 *
 * Description:  NPAC Conferencing System API functions for session 
 *               management 
 *
 * Author:       Tomasz Stachowiak, NPAC
 *
 * Date:         10-14-96
 *
 *********************************************************************/
#include <stdio.h>
#include <winsock.h>

#include "common.h"
#include "server.h"

FILE *log;

Participant member[MAX_MEMBERS];
int memberActive[MAX_MEMBERS];
int localHandle;

int server;


void (* inviteCallback)(void *);
void (* timeoutCallback)(void *);
void (* connectedCallback)(void *);
void (* acceptCallback)(void *);
void (* rejectCallback)(void *);
void (* deleteMemberCallback)(void *);
void (* addMemberCallback)(void *);
void (* startAppCallback)(void *);
void (* serverUsersCallback)(void *);

ManagerData managerData;


void InitMembers()
{
  int i;

  for(i=0; i<MAX_MEMBERS; i++)
    memberActive[i] = 0;
}

void InitCallbacks()
{
  inviteCallback = NULL;
  timeoutCallback = NULL;
  deleteMemberCallback = NULL;
  addMemberCallback = NULL;
  startAppCallback = NULL;
  serverUsersCallback = NULL;
  connectedCallback = NULL;
  acceptCallback = NULL;
  rejectCallback = NULL;
}

void AddMember(char *data)
{
  Participant *newmember;
  NCSparticipant part;

  newmember = (Participant *)data;
  PRINTF(log, "Manager: Member added locally %s@%d handle %d\n", newmember->user, newmember->host, newmember->handle);
  memberActive[newmember->handle] = 1;
  memcpy(&member[newmember->handle], newmember, sizeof(Participant));

  strcpy(part.user, newmember->user);
  if(getHostName(newmember->host, part.host) > 0)
    {
      PRINTF(log, "Manager: Member added locally %s@%s handle %d\n", part.user, part.host, newmember->handle);
      if(addMemberCallback != NULL)
	     addMemberCallback(&part);
    }
  else
  {
	  PRINTF(log, "Manager: Host unknown\n");
  }
}

void InvitationReceived(char *data)
{
  Participant *member;
  NCSparticipant part;

  member = (Participant *)data;
  strcpy(part.user, member->user);
  if(getHostName(member->host, part.host) > 0)
    {
      if(inviteCallback != NULL)
		inviteCallback(&part);
    }
  else
    PRINTF(log, "Manager: Host unknown\n");
}

void InvitationAccepted(char *data)
{
  Participant *member;
  NCSparticipant part;

  member = (Participant *)data;
  strcpy(part.user, member->user);
  if(getHostName(member->host, part.host) > 0)
    {
      if(acceptCallback != NULL)
	acceptCallback(&part);
    }
//  else
  //  PRINTF(log, "Manager: Host unknown\n");
}

void InvitationRejected(char *data)
{
  Participant *member;
  NCSparticipant part;

  member = (Participant *)data;
  strcpy(part.user, member->user);
  if(getHostName(member->host, part.host) > 0)
    {
      if(rejectCallback != NULL)
	rejectCallback(&part);
    }
  else
    PRINTF(log, "Manager: Host unknown\n");
}

void InvitationTimeout(char *data)
{
  Participant *member;
  NCSparticipant part;

  member = (Participant *)data;
  strcpy(part.user, member->user);
  if(getHostName(member->host, part.host) > 0)
    {
      if(timeoutCallback != NULL)
	timeoutCallback(&part);
    }
  else
    PRINTF(log, "Manager: Host unknown\n");
}

void InvitationDelivered(char *data)
{
  Participant *member;
  NCSparticipant part;

  member = (Participant *)data;
  strcpy(part.user, member->user);
  if(getHostName(member->host, part.host) > 0)
    {
      if(connectedCallback != NULL)
	      connectedCallback(&part);
    }
  else
    PRINTF(log, "Manager: Host unknown\n");
}

void DeleteMember(char *data)
{
  NCSparticipant part;
  int *no;

  no = (int *)data;
  memberActive[*no] = 0;

  strcpy(part.user, member[*no].user);
  if(getHostName(member[*no].host, part.host) > 0)
    {
      if(deleteMemberCallback != NULL)
	deleteMemberCallback(&part);
    }
  else
    PRINTF(log, "Manager: Host unknown\n");
}


int ncsSessionInitialize()
{
  static int first = 1;
  long hostid;
  WSADATA wsaData;	

#ifndef QUIET
  log = fopen("ncs.log", "w");
#endif

  PRINTF(log, "Init\n");
  if(WSAStartup(257, &wsaData) != 0)
	  PRINTF(log, "Manager: can't startup winsock\n");

  PRINTF(log, "WSA started up\n");
  if ((managerData.socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
      PRINTF(log, "Manager: can't open manage socket\n");
      return 0;
    }

  PRINTF(log, "Socket intialized\n");
 /*
  bzero(&managerData.addr, sizeof(managerData.addr));
  managerData.addr.sin_family = AF_INET;
  managerData.addr.sin_addr.s_addr = htonl(INADDR_ANY);
  managerData.addr.sin_port        = htons(MANAGER_PORT);

  if(bind(managerData.socket, &managerData.addr, sizeof(managerData.addr)) < 0)
    {
      PRINTF(stderr, "Manager: can't bind daemon socket\n");
      return 0;
    }
    */

  managerData.daemon_addr.sin_family = AF_INET;
  hostid = gethostid();
  PRINTF(log, "host id retrieved: %d\n", hostid);
  managerData.daemon_addr.sin_addr.s_addr = hostid;
  managerData.daemon_addr.sin_port        = htons(DAEMON_PORT);

  if(connect(managerData.socket, (struct sockaddr *)&managerData.daemon_addr, sizeof(managerData.daemon_addr)) < 0)
    { 
      PRINTF(log, "Manager: can not connect to engine\n");
      exit(0);
    }

  PRINTF(log, "Connected to engine\n");

  ncsEngineRequest(reqNewLocalSessionInit, NULL, 0);

  InitMembers();
  if(first)
    {
      InitCallbacks();
      first = 0;
    }

  //server = serverInit();
  server = 0;

  if(server)
	  serverAdd();

  PRINTF(log, "Initialized\n");
  return 1;
}

void ncsSessionCreate()
{
  ncsEngineRequest(reqLocalSessionCreate, NULL, 0);
  localHandle = 0;     /* what if session already created?!!! */
}


/* Registers working procedure running in the Xt background, 
   processing incoming control messages */
void ncsSessionRegisterWithXt(/*XtApp app*/)
{
}

/* Returns file descriptor to session control socket */
int ncsSessionGetFD(fd_set *set)
{
  FD_ZERO(set);
  FD_SET(managerData.socket, set);
  return managerData.socket;
}

/* Processes control messages coming to the control socket */
void ncsProcessIncoming(fd_set *set)
{
  if(FD_ISSET(managerData.socket, set))
    {
      ncsReadEngineMessage();
    }
}



void ncsReadEngineMessage()
{
  char *msg;
  MsgHeader header;
  int err;

  err = readMsgHeader(managerData.socket, &header);
  if(err < 0)
    {
      PRINTF(log, "Manager: data reading error\n");
      return;
    }
  if(err == 0)
    {
      PRINTF(log, "Manager: engine killed, exitting...\n");
      exit(0);
    }

  PRINTF(log, "Data length %d\n", header.dataLength);
  if(header.dataLength > 0)
    {
      msg = (char *)malloc(header.dataLength);
      err = readAll(managerData.socket, msg, header.dataLength);
      if(err < 0)
	{
	  PRINTF(log, "Manager: data reading error\n");
	  return;
	}
      if(err == 0)
	{
	  PRINTF(log, "Manager: engine killed, exitting...\n");
	  exit(0);
	}
    }
  else
    msg = NULL;
  
  ncsEngineMessageHandle(header.tag, msg, header.dataLength);
  
  free(msg);
}

void ncsEngineMessageHandle(short tag, char *msg, int len)
{
  InviteData *info;

  info = (InviteData *)msg;
  switch((LocalTag)tag)
    {
    case indLocalSessionInvited:
      PRINTF(log, "Manager: Invitation received\n");
      InvitationReceived(msg);
      break;

    case indLocalSessionAccept:
      PRINTF(log, "Manager: Invitation accepted\n");
      InvitationAccepted(msg);
      break;

    case indLocalSessionReject:
      PRINTF(log, "Manager: Invitation rejected\n");
      InvitationRejected(msg);
      break;
      
    case indLocalSessionMembers:
      break;

    case indLocalSessionAddMember:
      PRINTF(log, "Manager: add member\n");
      AddMember(msg);
      break;

    case indLocalSessionDeleteMember:
      PRINTF(log, "Manager: delete member\n");
      DeleteMember(msg);
      break;

    case indLocalSessionLocalMember:
      PRINTF(log, "Manager: local handle %d\n", *(int *)msg);
     localHandle = *(int *)msg;
      break;

    case indLocalSessionStartApp:
      PRINTF(log, "Manager: start application\n");
     if(startAppCallback != NULL)
	startAppCallback(msg);
      break;

    case indLocalSessionTimeout:
      PRINTF(log, "Manager: invitation timeout\n");
    InvitationTimeout(msg);
      break;

    case indLocalSessionConnected:
      PRINTF(log, "Manager: invitation delivered\n");
      InvitationDelivered(msg);
      break;

    case indServerUsers:
      PRINTF(log, "Manager: server users\n");
      serverUsersCallback(msg);
      break;

    default:
      PRINTF(log, "Manager: wrong message tag %d\n", (LocalTag)tag);
  }
}

void ncsSessionAddMember(char *user, char *host)
{
  Participant part;

  PRINTF(log, "Manager: invite member %s@%s\n", user, host);
  strcpy(part.user, user);
  part.host = getHostID(host);
  PRINTF(log, "Manager: invite member %s@%d\n", part.user, part.host);
  if(part.host != 0)
    ncsEngineRequest(reqLocalSessionAddMember, (char *)&part, sizeof(Participant));
  else
    PRINTF(log, "Manager: wrong host name\n");
}

void ncsSessionAcceptInvitation(NCSparticipant *part)
{
  Participant member;

  strcpy(member.user, part->user);
  member.host = getHostID(part->host);
  if(member.host != 0)
    {
      PRINTF(log, "Manager: join session %s@%s \n", part->user, part->host);
      ncsEngineRequest(reqLocalSessionJoin, (char *)&member, sizeof(Participant));
    }
  else
    PRINTF(log, "Manager: unknown host %s@%s \n", part->user, part->host);
}

void ncsSessionRejectInvitation(NCSparticipant *part)
{
  Participant member;

  strcpy(member.user, part->user);
  member.host = getHostID(part->host);
  if(member.host != 0)
    {
      PRINTF(log, "Manager: delete invitation from %s@%s \n", part->user, part->host);
      ncsEngineRequest(reqLocalSessionReject, (char *)&member, sizeof(Participant));
    }
  else
    PRINTF(log, "Manager: unknown host %s@%s \n", part->user, part->host);
}

void ncsSessionAddCallback(short tag, void (*function))
{
  switch(tag)
    {
    case indLocalSessionInvited:
      inviteCallback = (void (*)(void *))function;
      break;

    case indLocalSessionAddMember:
      addMemberCallback = (void (*)(void *))function;
      break;

    case indLocalSessionDeleteMember:
      deleteMemberCallback = (void (*)(void *))function;
      break;

    case indLocalSessionStartApp:
      startAppCallback = (void (*)(void *))function;
      break;

    case indLocalSessionTimeout:
      timeoutCallback = (void (*)(void *))function;
      break;

    case indLocalSessionConnected:
      connectedCallback = (void (*)(void *))function;
      break;

    case indLocalSessionAccept:
      acceptCallback = (void (*)(void *))function;
      break;

    case indLocalSessionReject:
      rejectCallback = (void (*)(void *))function;
      break;

    case indServerUsers:
      serverUsersCallback = (void (*)(void *))function;
      break;

    default:
      PRINTF(log, "Manager: Unknown control callback type\n");
    }
}

void ncsSessionQuit()
{
  if(server)
	serverDel();
  ncsEngineRequest(reqLocalSessionQuit, NULL, 0);
  closesocket(managerData.socket);
  WSACleanup();
#ifndef QUIET
  fclose(log);
#endif
}

void ncsSessionQueryMembers()
{
  ncsEngineRequest(reqLocalSessionGetMembers, NULL, 0);
}


void ncsSessionSendToApp(int port, short type, char *msg, int len)
{
  AppData *data;
  int size;

  if(len == 0)
    {
      size = sizeof(AppData);
      data = (AppData *)malloc(size);
      data->data[0] = 0;
    }
  else
    {
      size = sizeof(AppData) + len - 1;
      data = (AppData *)malloc(size);
      memcpy(data->data, data, len);
    }
    

  data->port = port;
  PRINTF(log, "Manager: Send to app port %d, msg size %d\n", data->port, size);
  data->tag = type;
  memcpy(data->data, msg, len);
  ncsEngineRequest(reqLocalSessionSendToApp, (char *)data, size);
  free(data);
}

void ncsSessionStartApp(int port)
{
  ncsEngineRequest(reqLocalSessionStartApp, (char *)&port, sizeof(int));
}

void ncsSessionReset()
{ 
  ncsEngineRequest(reqLocalSessionReset, NULL, 0);
}

int ncsSessionSaveInit()
{
  ncsEngineRequest(reqLocalSessionSaveInit, NULL, 0);
  return 1;
}

void ncsSessionSaveClose()
{
  ncsEngineRequest(reqLocalSessionSaveClose, NULL, 0);
}

int ncsSessionRestoreInit()
{
  ncsEngineRequest(reqLocalSessionRestoreInit, NULL, 0);
  return 1;
}

void ncsSessionRestoreClose()
{
  ncsEngineRequest(reqLocalSessionRestoreClose, NULL, 0);
}





